@@ -241,7 +241,7 @@ irc_topic (server *serv, char *channel, char *topic)
 }
 
 static void
-irc_list_channels (server *serv, char *arg)
+irc_list_channels (server *serv, char *arg, int min_users)
 {
 	if (arg[0])
 	{
@@ -250,8 +250,7 @@ irc_list_channels (server *serv, char *arg)
 	}
 
 	if (serv->use_listargs)
-								/* 1234567890123456 */
-		tcp_send_len (serv, "LIST >0,<10000\r\n", 16);
+		tcp_sendf (serv, "LIST >%d,<10000\r\n", min_users - 1);
 	else
 		tcp_send_len (serv, "LIST\r\n", 6);
 }
@@ -342,7 +341,8 @@ process_numeric (session * sess, int n,
 		}
 
 		/* use /NICKSERV */
-		if (strcasecmp (word[7], "DALnet") == 0)
+		if (strcasecmp (word[7], "DALnet") == 0 ||
+			 strcasecmp (word[7], "BRASnet") == 0)
 			serv->nickservtype = 1;
 
 		/* use /NS */
@@ -627,7 +627,10 @@ process_numeric (session * sess, int n,
 	case 349:	/* end of exemption list */
 		sess = find_channel (serv, word[4]);
 		if (!sess)
+		{
 			sess = serv->front_session;
+			goto def;
+		}
 		if (!fe_is_banwindow (sess))
 			goto def;
 		fe_ban_list_end (sess, TRUE);
@@ -650,7 +653,10 @@ process_numeric (session * sess, int n,
 	case 368:
 		sess = find_channel (serv, word[4]);
 		if (!sess)
+		{
 			sess = serv->front_session;
+			goto def;
+		}
 		if (!fe_is_banwindow (sess))
 			goto def;
 		fe_ban_list_end (sess, FALSE);
@@ -918,7 +924,7 @@ process_named_msg (session *sess, char *type, char *word[], char *word_eol[])
 							flood_check (nick, ip, serv, sess, 0);
 						if (strncasecmp (text, "DCC ", 4) == 0)
 							/* redo this with handle_quotes TRUE */
-							process_data_init (word[1], word_eol[1], word, word_eol, TRUE);
+							process_data_init (word[1], word_eol[1], word, word_eol, TRUE, FALSE);
 						ctcp_handle (sess, to, nick, text, word, word_eol, id);
 					} else
 					{
@@ -1006,7 +1012,7 @@ irc_inline (server *serv, char *buf, int len)
 	if (buf[0] == ':')
 	{
 		/* split line into words and words_to_end_of_line */
-		process_data_init (pdibuf, buf, word, word_eol, FALSE);
+		process_data_init (pdibuf, buf, word, word_eol, FALSE, FALSE);
 
 		/* find a context for this message */
 		if (is_channel (serv, word[3]))
@@ -1028,7 +1034,7 @@ irc_inline (server *serv, char *buf, int len)
 
 	} else
 	{
-		process_data_init (pdibuf, buf, word, word_eol, FALSE);
+		process_data_init (pdibuf, buf, word, word_eol, FALSE, FALSE);
 		word[0] = type = word[1];
 		if (plugin_emit_server (sess, type, word, word_eol))
 			goto xit;
