@@ -166,6 +166,7 @@ static const struct defaultserver def[] =
 	{0,			"irc.deltaanime.net"},
 
 	{"EFnet",	0},
+	{0,			"irc.blackened.com"},
 	{0,			"irc.Prison.NET"},
 	{0,			"irc.Qeast.net"},
 	{0,			"irc.efnet.pl"},
@@ -991,6 +992,13 @@ servlist_check_encoding (char *charset)
 	if (c)
 		c[0] = 0;
 
+	if (!strcasecmp (charset, "IRC")) /* special case */
+	{
+		if (c)
+			c[0] = ' ';
+		return TRUE;
+	}
+
 	gic = g_iconv_open (charset, "UTF-8");
 
 	if (c)
