@@ -228,7 +228,7 @@ log_create_pathname (char *servname, char *channame, char *netname)
 	snprintf (fname, sizeof (fname), "%s/xchatlogs/%s", get_xdir_utf8 (), fnametime);
 
 	/* now we need it in FileSystem encoding */
-	fs = g_filename_from_utf8 (fname, -1, 0, 0, 0);
+	fs = xchat_filename_from_utf8 (fname, -1, 0, 0, 0);
 
 	/* create all the subdirectories */
 	if (fs)
@@ -363,40 +363,142 @@ log_write (session *sess, char *text)
 	}
 }
 
+/* converts a CP1252/ISO-8859-1(5) hybrid to UTF-8                           */
+/* Features: 1. It never fails, all 00-FF chars are converted to valid UTF-8 */
+/*           2. Uses CP1252 in the range 80-9f because ISO doesn't have any- */
+/*              thing useful in this range and it helps us receive from mIRC */
+/*           3. The five undefined chars in CP1252 80-9f are replaced with   */
+/*              ISO-8859-15 control codes.                                   */
+/*           4. Handles 0xa4 as a Euro symbol ala ISO-8859-15.               */
+/*           5. Uses ISO-8859-1 (which matches CP1252) for everything else.  */
+/*           6. This routine measured 3x faster than g_convert :)            */
+
+static unsigned char *
+iso_8859_1_to_utf8 (unsigned char *text, int len, gsize *bytes_written)
+{
+	unsigned int idx;
+	unsigned char *res, *output;
+	static const unsigned short lowtable[] = /* 74 byte table for 80-a4 */
+	{
+	/* compressed utf-8 table: if the first byte's 0x20 bit is set, it
+	   indicates a 2-byte utf-8 sequence, otherwise prepend a 0xe2. */
+		0x82ac, /* 80 Euro. CP1252 from here on... */
+		0xe281, /* 81 NA */
+		0x809a, /* 82 */
+		0xe692, /* 83 */
+		0x809e, /* 84 */
+		0x80a6, /* 85 */
+		0x80a0, /* 86 */
+		0x80a1, /* 87 */
+		0xeb86, /* 88 */
+		0x80b0, /* 89 */
+		0xe5a0, /* 8a */
+		0x80b9, /* 8b */
+		0xe592, /* 8c */
+		0xe28d, /* 8d NA */
+		0xe5bd, /* 8e */
+		0xe28f, /* 8f NA */
+		0xe290, /* 90 NA */
+		0x8098, /* 91 */
+		0x8099, /* 92 */
+		0x809c, /* 93 */
+		0x809d, /* 94 */
+		0x80a2, /* 95 */
+		0x8093, /* 96 */
+		0x8094, /* 97 */
+		0xeb9c, /* 98 */
+		0x84a2, /* 99 */
+		0xe5a1, /* 9a */
+		0x80ba, /* 9b */
+		0xe593, /* 9c */
+		0xe29d, /* 9d NA */
+		0xe5be, /* 9e */
+		0xe5b8, /* 9f */
+		0xe2a0, /* a0 */
+		0xe2a1, /* a1 */
+		0xe2a2, /* a2 */
+		0xe2a3, /* a3 */
+		0x82ac  /* a4 ISO-8859-15 Euro. */
+	};
+
+	if (len == -1)
+		len = strlen (text);
+
+	/* worst case scenario: every byte turns into 3 bytes */
+	res = output = g_malloc ((len * 3) + 1);
+	if (!output)
+		return NULL;
+
+	while (len)
+	{
+		if (G_LIKELY (*text < 0x80))
+		{
+			*output = *text;	/* ascii maps directly */
+		}
+		else if (*text <= 0xa4)	/* 80-a4 use a lookup table */
+		{
+			idx = *text - 0x80;
+			if (lowtable[idx] & 0x2000)
+			{
+				*output++ = (lowtable[idx] >> 8) & 0xdf; /* 2 byte utf-8 */
+				*output = lowtable[idx] & 0xff;
+			}
+			else
+			{
+				*output++ = 0xe2;	/* 3 byte utf-8 */
+				*output++ = (lowtable[idx] >> 8) & 0xff;
+				*output = lowtable[idx] & 0xff;
+			}
+		}
+		else if (*text < 0xc0)
+		{
+			*output++ = 0xc2;
+			*output = *text;
+		}
+		else
+		{
+			*output++ = 0xc3;
+			*output = *text - 0x40;
+		}
+		output++;
+		text++;
+		len--;
+	}
+	*output = 0;	/* terminate */
+	*bytes_written = output - res;
+
+	return res;
+}
+
 char *
 text_validate (char **text, int *len)
 {
 	char *utf;
 	gsize utf_len;
-	GError *error = NULL;
 
 	/* valid utf8? */
 	if (g_utf8_validate (*text, *len, 0))
 		return NULL;
 
+#ifdef WIN32
+	if (GetACP () == 1252) /* our routine is better than iconv's 1252 */
+#else
 	if (prefs.utf8_locale)
+#endif
 		/* fallback to iso-8859-1 */
-		utf = g_convert (*text, *len, "UTF-8", "ISO-8859-1", 0, &utf_len, &error);
+		utf = iso_8859_1_to_utf8 (*text, *len, &utf_len);
 	else
 	{
 		/* fallback to locale */
 		utf = g_locale_to_utf8 (*text, *len, 0, &utf_len, NULL);
 		if (!utf)
-			utf = g_convert (*text, *len, "UTF-8", "ISO-8859-1", 0, &utf_len, &error);
+			utf = iso_8859_1_to_utf8 (*text, *len, &utf_len);
 	}
 
 	if (!utf) 
 	{
-		if (error)
-		{
-			*text = g_strdup_printf ("\0034ICONV\017 %s\n", error->message);
-			*len = strlen (*text);
-			g_error_free (error);
-		} else
-		{
-			*text = g_strdup ("%INVALID%");
-			*len = 9;
-		}
+		*text = g_strdup ("%INVALID%");
+		*len = 9;
 	} else
 	{
 		*text = utf;
@@ -821,12 +923,13 @@ static char * const pevt_connect_help[] = {
 };
 
 static char * const pevt_sconnect_help[] = {
-	N_("PID"),
+	"PID"
 };
 
 static char * const pevt_generic_nick_help[] = {
 	N_("Nickname"),
-	N_("Server Name")
+	N_("Server Name"),
+	N_("Network")
 };
 
 static char * const pevt_chanmodes_help[] = {
@@ -1011,8 +1114,8 @@ pevent_load_defaults ()
 		if (pntevts_text[i])
 			free (pntevts_text[i]);
 
-		/* don't gettext() the blank ones */
-		if (i == XP_TE_OPENDIALOG || i == XP_TE_BEEP)
+		/* make-te.c sets this 128 flag (DON'T call gettext() flag) */
+		if (te[i].num_args & 128)
 			pntevts_text[i] = strdup (te[i].def);
 		else
 			pntevts_text[i] = strdup (_(te[i].def));
@@ -1022,7 +1125,7 @@ pevent_load_defaults ()
 void
 pevent_make_pntevts ()
 {
-	int i, m, len;
+	int i, m;
 	char out[1024];
 
 	for (i = 0; i < NUM_XP; i++)
@@ -1035,9 +1138,11 @@ pevent_make_pntevts ()
 						 _("Error parsing event %s.\nLoading default."), te[i].name);
 			fe_message (out, FE_MSG_WARN);
 			free (pntevts_text[i]);
-			len = strlen (te[i].def) + 1;
-			pntevts_text[i] = malloc (len);
-			memcpy (pntevts_text[i], te[i].def, len);
+			/* make-te.c sets this 128 flag (DON'T call gettext() flag) */
+			if (te[i].num_args & 128)
+				pntevts_text[i] = strdup (te[i].def);
+			else
+				pntevts_text[i] = strdup (_(te[i].def));
 			if (pevt_build_string (pntevts_text[i], &(pntevts[i]), &m) != 0)
 			{
 				fprintf (stderr,
@@ -1204,7 +1309,7 @@ pevent_load (char *filename)
 static void
 pevent_check_all_loaded ()
 {
-	int i, len;
+	int i;
 
 	for (i = 0; i < NUM_XP; i++)
 	{
@@ -1213,9 +1318,11 @@ pevent_check_all_loaded ()
 			/*printf ("%s\n", te[i].name);
 			snprintf(out, sizeof(out), "The data for event %s failed to load. Reverting to defaults.\nThis may be because a new version of XChat is loading an old config file.\n\nCheck all print event texts are correct", evtnames[i]);
 			   gtkutil_simpledialog(out); */
-			len = strlen (te[i].def) + 1;
-			pntevts_text[i] = malloc (len);
-			memcpy (pntevts_text[i], te[i].def, len);
+			/* make-te.c sets this 128 flag (DON'T call gettext() flag) */
+			if (te[i].num_args & 128)
+				pntevts_text[i] = strdup (te[i].def);
+			else
+				pntevts_text[i] = strdup (_(te[i].def));
 		}
 	}
 }
@@ -1237,7 +1344,7 @@ load_text_events ()
 	1) if prefs.stripcolor is set, filter all style control codes from arguments
 	2) always strip \010 (ATTR_HIDDEN) from arguments: it is only for use in the format string itself
 */
-#define ARG_FLAG(argn) (1 << argn)
+#define ARG_FLAG(argn) (1 << (argn))
 
 void
 format_event (session *sess, int index, char **args, char *o, int sizeofo, unsigned int stripcolor_args)
@@ -1246,7 +1353,7 @@ format_event (session *sess, int index, char **args, char *o, int sizeofo, unsig
 	char *i, *ar, d, a, done_all = FALSE;
 
 	i = pntevts[index];
-	numargs = te[index].num_args;
+	numargs = te[index].num_args & 0x7f;
 
 	oi = ii = len = d = a = 0;
 	o[0] = 0;
@@ -1676,7 +1783,7 @@ sound_play (const char *file, gboolean quiet)
 	}
 	wavfile[sizeof (wavfile) - 1] = 0;	/* ensure termination */
 
-	file_fs = g_filename_from_utf8 (wavfile, -1, 0, 0, 0);
+	file_fs = xchat_filename_from_utf8 (wavfile, -1, 0, 0, 0);
 	if (!file_fs)
 		return;
 
