@@ -1,4 +1,4 @@
-#include "../../config.h"
+#include "config.h"
 
 #include <glib/gslist.h>
 #include <glib/glist.h>
@@ -81,9 +81,6 @@ void *xchat_realloc (char *old, int len, char *file, int line);
 #define USERNAMELEN 10
 #define HIDDEN_CHAR	8			/* invisible character for xtext */
 
-#define safe_strcpy(dest,src,len)	{strncpy((dest),(src),(len)); \
-												(dest)[len-1] = 0;}
-
 #if defined(ENABLE_NLS) && !defined(_)
 #  include <libintl.h>
 #  define _(x) gettext(x)
@@ -175,6 +172,7 @@ struct xchatprefs
 	int mainwindow_height;
 	int completion_sort;
 	int gui_win_state;
+	int gui_url_mod;
 	int gui_usermenu;
 	int gui_join_dialog;
 	int dialog_left;
@@ -231,7 +229,8 @@ struct xchatprefs
 	unsigned int beepmsg;
 	unsigned int beepchans;
 	unsigned int beephilight;
-	unsigned int flash_hilight;
+	unsigned int input_flash_hilight;
+	unsigned int input_flash_priv;
 	unsigned int truncchans;
 	unsigned int privmsgtab;
 	unsigned int irc_join_delay;
@@ -338,10 +337,11 @@ typedef struct session
 	int end_of_names:1;
 	int doing_who:1;		/* /who sent on this channel */
 	/* these are in the bottom-right menu */
-	int hide_join_part:1;	/* hide join & part messages? */
-	int beep:1;				/* beep enabled? */
-	int color_paste:1;
+	unsigned int hide_join_part:1;	/* hide join & part messages? */
+	unsigned int beep:1;				/* beep enabled? */
+	unsigned int color_paste:1;
 	int done_away_check:1;	/* done checking for away status changes */
+	unsigned int lastlog_regexp:1;	/* this is a lastlog and using regexp */
 } session;
 
 struct msproxy_state_t
@@ -386,7 +386,7 @@ typedef struct server
 	void (*p_action)(struct server *, char *channel, char *act);
 	void (*p_notice)(struct server *, char *channel, char *text);
 	void (*p_topic)(struct server *, char *channel, char *topic);
-	void (*p_list_channels)(struct server *, char *arg);
+	void (*p_list_channels)(struct server *, char *arg, int min_users);
 	void (*p_change_nick)(struct server *, char *new_nick);
 	void (*p_names)(struct server *, char *channel);
 	void (*p_ping)(struct server *, char *to, char *timestring);
@@ -436,6 +436,7 @@ typedef struct server
 	time_t next_send;						/* cptr->since in ircu */
 	time_t prev_now;					/* previous now-time */
 	int sendq_len;						/* queue size */
+	int lag;								/* milliseconds */
 
 	struct session *front_session;	/* front-most window/tab */
 	struct session *server_session;	/* server window/tab */
@@ -475,6 +476,7 @@ typedef struct server
 	unsigned int have_idmsg:1;	/* freenode's IDENTIFY-MSG */
 	unsigned int have_except:1;	/* ban exemptions +e */
 	unsigned int using_cp1255:1;	/* encoding is CP1255/WINDOWS-1255? */
+	unsigned int using_irc:1;		/* encoding is "IRC" (CP1252/UTF-8 hybrid)? */
 	int use_who:1;				/* whether to use WHO command to get dcc_ip */
 #ifdef USE_OPENSSL
 	int use_ssl:1;					  /* is server SSL capable? */
@@ -491,7 +493,7 @@ struct commands
 	cmd_callback callback;
 	char needserver;
 	char needchannel;
-	char handle_quotes;
+	gint16 handle_quotes;
 	char *help;
 };
 
@@ -514,4 +516,12 @@ struct popup
 /* CL: get a random int in the range [0..n-1]. DON'T use rand() % n, it gives terrible results. */
 #define RAND_INT(n) ((int)(rand() / (RAND_MAX + 1.0) * (n)))
 
+#if defined(WIN32) && GLIB_CHECK_VERSION(2,4,0)
+#define xchat_filename_from_utf8 g_locale_from_utf8
+#define xchat_filename_to_utf8 g_locale_to_utf8
+#else
+#define xchat_filename_from_utf8 g_filename_from_utf8
+#define xchat_filename_to_utf8 g_filename_to_utf8
+#endif
+
 #endif
