@@ -1,7 +1,7 @@
 /*
  * fe-gnome.c - main frontend implementation
  *
- * Copyright (C) 2004-2006 xchat-gnome team
+ * Copyright (C) 2004-2007 xchat-gnome team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,15 +32,18 @@
 #include "userlist-gui.h"
 #include "preferences.h"
 #include "setup-dialog.h"
+#include "migration.h"
 
 #include "conversation-panel.h"
 #include "status-bar.h"
 #include "topic-label.h"
 #include "connect-dialog.h"
+#include "text-entry.h"
+#include "find-bar.h"
 
 #include "palette.h"
 #include "preferences-page-plugins.h"
-#include "channel-list.h"
+#include "channel-list-window.h"
 #include "util.h"
 #include "plugins.h"
 #include "../common/xchat.h"
@@ -50,6 +53,7 @@
 #include "../common/fe.h"
 #include "../common/util.h"
 #include "../common/cfgfiles.h"
+#include "../common/plugin.h"
 
 static gboolean  opt_version   = FALSE;
 static gboolean  opt_noplugins = FALSE;
@@ -60,6 +64,7 @@ static GOptionEntry entries[] = {
 	{"no-auto",    'a', 0, G_OPTION_ARG_NONE,     &arg_dont_autoconnect, N_("Don't auto-connect to servers"),                   NULL},
 	{"no-plugins", 'n', 0, G_OPTION_ARG_NONE,     &opt_noplugins,        N_("Don't auto-load plugins"),                         NULL},
 	{"url",        'u', 0, G_OPTION_ARG_STRING,   &arg_url,              N_("Open an irc:// url"),                              "irc://server:port/channel"},
+	{"existing",   'e', 0, G_OPTION_ARG_NONE,     &arg_existing,         N_("Open URL in an existing XChat-GNOME instance"),    NULL},
 	{"version",    'v', 0, G_OPTION_ARG_NONE,     &opt_version,          N_("Show version information"),                        NULL},
 	{ NULL }
 };
@@ -91,28 +96,15 @@ fe_args (int argc, char *argv[])
 	textdomain (GETTEXT_PACKAGE);
 #endif
 
-	context = g_option_context_new ("");
+	context = g_option_context_new (NULL);
 
-#ifdef HAVE_LIBGNOME_214
 	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
-	gui.program = gnome_program_init (PACKAGE_NAME, PACKAGE_VERSION,
+	gui.program = gnome_program_init (PACKAGE, VERSION,
 	                                  LIBGNOMEUI_MODULE, argc, argv,
 	                                  GNOME_PARAM_GOPTION_CONTEXT, context,
-	                                  GNOME_PARAM_HUMAN_READABLE_NAME, _("IRC Chat"),
+	                                  GNOME_PARAM_HUMAN_READABLE_NAME, "IRC Chat",
 	                                  GNOME_PROGRAM_STANDARD_PROPERTIES,
 	                                  NULL);
-#else
-	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
-	g_option_context_add_group (context, gtk_get_option_group (TRUE));
-	g_option_context_parse (context, &argc, &argv, &error);
-	g_option_context_free (context);
-
-	gui.program = gnome_program_init (PACKAGE_NAME, PACKAGE_VERSION,
-	                                  LIBGNOMEUI_MODULE, argc, argv,
-	                                  GNOME_PARAM_HUMAN_READABLE_NAME, _("IRC Chat"),
-	                                  GNOME_PROGRAM_STANDARD_PROPERTIES,
-	                                  NULL);
-#endif
 
 	if (error) {
 		fprintf (stderr, _("xchat-gnome: %s\nTry `xchat-gnome --help' for more information\n"), error->message);
@@ -125,8 +117,9 @@ fe_args (int argc, char *argv[])
 		return 0;
 	}
 
-	if (opt_cfgdir)
+	if (opt_cfgdir) {
 		xdir_fs = opt_cfgdir;
+	}
 
 	setup_sm (argc, argv);
 
@@ -136,11 +129,8 @@ fe_args (int argc, char *argv[])
 void
 fe_init (void)
 {
-	gchar *accel_map;
 	GConfClient *client;
 
-	gnome_vfs_init ();
-
 	client = gconf_client_get_default ();
 	gconf_client_add_dir (client, "/apps/xchat", GCONF_CLIENT_PRELOAD_NONE, NULL);
 	g_object_unref (client);
@@ -149,10 +139,12 @@ fe_init (void)
 	gui.quit = FALSE;
 	servlist_init ();
 	palette_init ();
-	if (!preferences_exist ())
+	run_migrations ();
+	if (!preferences_exist ()) {
 		run_setup_dialog ();
-	else
+	} else {
 		set_version ();
+	}
 	load_preferences ();
 	initialize_gui_1 ();
 	initialize_gui_2 ();
@@ -162,21 +154,18 @@ fe_init (void)
 	prefs.use_server_tab = TRUE;
 	prefs.notices_tabs = FALSE;
 	prefs.servernotice = TRUE;
+	prefs.slist_skip = FALSE;
 
 	/* If we don't have a specific DCC IP address, force get-from-server */
-	if (strlen (prefs.dcc_ip_str) == 0)
+	if (strlen (prefs.dcc_ip_str) == 0) {
 		prefs.ip_from_server = TRUE;
+	}
 
 	/* Don't allow the core to autoload plugins. We use our own
 	 * method for autoloading.
 	 */
 	arg_skip_plugins = 1;
 
-	accel_map = g_strdup_printf ("%s%c%s", get_xdir_fs (), G_DIR_SEPARATOR, "accelerator_map");
-	if (g_file_test (accel_map, G_FILE_TEST_EXISTS))
-		gtk_accel_map_load (accel_map);
-	g_free (accel_map);
-
 	if (not_autoconnect ()) {
 		ConnectDialog *cd;
 
@@ -194,9 +183,7 @@ fe_main (void)
 {
 	gtk_main ();
 
-#ifdef HAVE_LIBGNOME_214
 	g_object_unref (gui.program);
-#endif
 
 	/* sleep for 3 seconds so any QUIT messages are not lost. The  */
 	/* GUI is closed at this point, so the user doesn't even know! */
@@ -204,18 +191,15 @@ fe_main (void)
 	/* FIXME: this is a crappy hack copied from fe-gtk. There's got
 	 * to be a way to ensure that the quit messages get sent before
 	 * we finish */
-	if (prefs.wait_on_exit)
+	if (prefs.wait_on_exit) {
 		sleep (3);
+	}
 }
 
 void
 fe_cleanup (void)
 {
-	gchar *accel_map;
-
-	accel_map = g_strdup_printf ("%s%c%s", get_xdir_fs (), G_DIR_SEPARATOR, "accelerator_map");
-	gtk_accel_map_save (accel_map);
-	g_free (accel_map);
+	gnome_accelerators_sync ();
 }
 
 void
@@ -241,25 +225,34 @@ fe_new_window (struct session *sess, int focus)
 {
 	static gboolean loaded = FALSE;
 
-	if (focus) {
-		gui.current_session = sess;
-		topic_label_set_topic (TOPIC_LABEL (gui.topic_label), sess, sess->topic);
+	conversation_panel_add_session (CONVERSATION_PANEL (gui.conversation_panel), sess, (gboolean) focus);
+
+	switch (sess->type) {
+	case SESS_SERVER:
+		navigation_model_add_server (gui.tree_model, sess);
+		break;
+	case SESS_CHANNEL:
+	case SESS_DIALOG:
+		navigation_model_add_channel (gui.tree_model, sess);
 	}
 
-	conversation_panel_add_session (CONVERSATION_PANEL (gui.conversation_panel), sess, (gboolean) focus);
-	if (sess->type == SESS_SERVER) {
-		navigation_tree_create_new_network_entry (gui.server_tree, sess);
-	} else if (sess->type == SESS_CHANNEL) {
-		navigation_tree_create_new_channel_entry (gui.server_tree, sess, (gboolean) focus);
+	/*
 	} else if (sess->type == SESS_DIALOG) {
 		struct User *user = NULL;
 
 		user = userlist_find_global (sess->server, sess->channel);
 
 		navigation_tree_create_new_channel_entry (gui.server_tree, sess, (gboolean) focus);
-		if (user)
+		if (user) {
 			topic_label_set_topic (TOPIC_LABEL (gui.topic_label), sess, user->hostname);
+		}
 	}
+	*/
+
+	if (focus) {
+		navigation_tree_select_session (gui.server_tree, sess);
+	}
+
 #ifdef USE_PLUGIN
 	if (!(opt_noplugins || loaded)) {
 		loaded = TRUE;
@@ -294,12 +287,15 @@ fe_input_add (int sok, int flags, void *func, void *data)
 
 	channel = g_io_channel_unix_new (sok);
 
-	if (flags & FIA_READ)
+	if (flags & FIA_READ) {
 		type |= G_IO_IN | G_IO_HUP | G_IO_ERR;
-	if (flags & FIA_WRITE)
+	}
+	if (flags & FIA_WRITE) {
 		type |= G_IO_OUT | G_IO_ERR;
-	if (flags & FIA_EX)
+	}
+	if (flags & FIA_EX) {
 		type |= G_IO_PRI;
+	}
 
 	tag = g_io_add_watch (channel, type, (GIOFunc) func, data);
 	g_io_channel_unref (channel);
@@ -329,12 +325,11 @@ void
 fe_set_hilight (struct session *sess)
 {
 	navigation_model_set_hilight (gui.tree_model, sess);
-	if (!gtk_window_is_active (GTK_WINDOW (gui.main_window)))
-		gtk_window_set_urgency_hint (GTK_WINDOW (gui.main_window), TRUE);
+	fe_flash_window (sess);
 }
 
 void
-fe_set_tab_color (struct session *sess, int col, int flash)
+fe_set_tab_color (struct session *sess, int col)
 {
 	/* FIXME: implement */
 }
@@ -409,11 +404,25 @@ fe_text_clear (struct session *sess)
 void
 fe_close_window (struct session *sess)
 {
-	if (!gui.quit) {
-		if (sess->type == SESS_CHANNEL)
-			navigation_tree_remove_channel (gui.server_tree, sess);
-		else
-			navigation_tree_remove_server (gui.server_tree, sess);
+	/*
+	 * There's really no point in doing all of this if the user is
+	 * quitting the app.  It makes it slow (as they watch individual
+	 * channels and servers disappear), and the OS is about to free
+	 * everything much more efficiently than we ever could.
+	 *
+	 * If we ever choose to run on Windows ME, this could be a problem :)
+	 */
+	if (gui.quit) {
+		session_free (sess);
+		return;
+	}
+
+	navigation_tree_remove_session (gui.server_tree, sess);
+	conversation_panel_remove_session (CONVERSATION_PANEL (gui.conversation_panel), sess);
+	topic_label_remove_session (TOPIC_LABEL (gui.topic_label), sess);
+	text_entry_remove_session (TEXT_ENTRY (gui.text_entry), sess);
+	if (sess->type == SESS_SERVER) {
+		status_bar_remove_server (STATUS_BAR (gui.status_bar), sess->server);
 	}
 
 	session_free (sess);
@@ -437,9 +446,11 @@ fe_print_text (struct session *sess, char *text)
 	conversation_panel_print (CONVERSATION_PANEL (gui.conversation_panel), sess, text, prefs.indent_nicks);
 	sess->new_data = TRUE;
 	navigation_model_set_hilight (gui.tree_model, sess);
-	if (sess->nick_said)
-		if (!gtk_window_is_active (GTK_WINDOW (gui.main_window)))
+	if (sess->nick_said) {
+		if (!gtk_window_is_active (GTK_WINDOW (gui.main_window))) {
 			gtk_window_set_urgency_hint (GTK_WINDOW (gui.main_window), TRUE);
+		}
+	}
 }
 
 void
@@ -496,8 +507,8 @@ fe_dcc_add (struct DCC *dcc)
 			                       _("_Accept"),
 			                       GTK_RESPONSE_ACCEPT);
 			gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-				_("%s is attempting to create a direct chat. Do you wish to accept the connection?"),
-				dcc->nick);
+			                                          _("%s is attempting to create a direct chat. Do you wish to accept the connection?"),
+			                                          dcc->nick);
 
 			response = gtk_dialog_run (GTK_DIALOG (dialog));
 			if (response == GTK_RESPONSE_ACCEPT) {
@@ -559,14 +570,20 @@ fe_dcc_open_chat_win (int passive)
 void
 fe_clear_channel (struct session *sess)
 {
-	navigation_model_set_disconn (gui.tree_model, sess);
+	navigation_model_set_disconnected (gui.tree_model, sess);
 }
 
 void
 fe_session_callback (struct session *sess)
 {
-	/* this frees things */
-	/* FIXME: implement */
+	if (sess->type == SESS_SERVER) {
+		status_bar_remove_server (STATUS_BAR (gui.status_bar), sess->server);
+	}
+
+	conversation_panel_remove_session (CONVERSATION_PANEL (gui.conversation_panel), sess);
+	topic_label_remove_session (TOPIC_LABEL (gui.topic_label), sess);
+	text_entry_remove_session (TEXT_ENTRY (gui.text_entry), sess);
+	userlist_erase (u, sess);
 }
 
 void
@@ -608,7 +625,7 @@ fe_dcc_send_filereq (struct session *sess, char *nick, int maxcps, int passive)
 void
 fe_set_channel (struct session *sess)
 {
-    navigation_tree_set_channel_name (gui.server_tree, sess);
+	navigation_model_update (gui.tree_model, sess);
 }
 
 void
@@ -649,8 +666,9 @@ fe_beep (void)
 }
 
 void
-fe_lastlog (session * sess, session * lastlog_sess, char *sstr)
+fe_lastlog (session * sess, session * lastlog_sess, char *sstr, gboolean regexp)
 {
+	/* FIXME: handle regexp */
 	conversation_panel_lastlog (CONVERSATION_PANEL (gui.conversation_panel), sess, lastlog_sess, sstr);
 }
 
@@ -660,12 +678,14 @@ fe_set_lag (server *serv, int lag)
 	unsigned long now;
 	float seconds;
 
-	if (gui.quit)
+	if (gui.quit) {
 		return;
+	}
 
 	if (lag == -1) {
-		if (!serv->lag_sent)
+		if (!serv->lag_sent) {
 			return;
+		}
 		now = make_ping_time ();
 		seconds = (now - serv->lag_sent) / 1000000.0f;
 	} else {
@@ -678,8 +698,9 @@ fe_set_lag (server *serv, int lag)
 void
 fe_set_throttle (server * serv)
 {
-	if (gui.quit)
+	if (gui.quit) {
 		return;
+	}
 
 	status_bar_set_queue (STATUS_BAR (gui.status_bar), serv, serv->sendq_len);
 }
@@ -734,10 +755,12 @@ fe_gui_info (session * sess, int info_type)
 {
 	switch (info_type) {
 	case 0:
-		if (!GTK_WIDGET_VISIBLE (GTK_WINDOW (gui.main_window)))
+		if (!GTK_WIDGET_VISIBLE (GTK_WINDOW (gui.main_window))) {
 			return 2;
-		if (gtk_window_is_active (GTK_WINDOW (gui.main_window)))
+		}
+		if (gtk_window_is_active (GTK_WINDOW (gui.main_window))) {
 			return 1;
+		}
 		return 0;
 		break;
 	}
@@ -899,14 +922,16 @@ fe_get_int (char *msg, int def, void *callback, void *userdata)
 void
 fe_open_url (const char *url)
 {
+	GdkScreen *screen;
 	GError *err = NULL;
 
+	screen = gtk_widget_get_screen (gui.main_window);
 	if (strstr (url, "://") == NULL) {
 		gchar *newword = g_strdup_printf ("http://%s", url);
-		gnome_url_show (newword, &err);
+		gnome_url_show_on_screen (newword, screen, &err);
 		g_free (newword);
 	} else {
-		gnome_url_show (url, &err);
+		gnome_url_show_on_screen (url, screen, &err);
 	}
 
 	if (err != NULL) {
@@ -923,8 +948,15 @@ fe_menu_del (menu_entry *entry)
 	/* FIXME: implement? */
 }
 
-void
+char *
 fe_menu_add (menu_entry *entry)
+{
+	/* FIXME: implement? */
+	return NULL;
+}
+
+void
+fe_set_color_paste (session *sess, int status)
 {
 	/* FIXME: implement? */
 }
@@ -951,11 +983,24 @@ fe_server_event (server *serv, int type, int arg)
 		sess = list->data;
 		if (sess->server == serv) {
 			switch (type) {
-				case FE_SE_LOGGEDIN:
-					if (arg == 0)
-						/* No auto-join channels */
-						create_channel_list (sess);
-					break;
+			case FE_SE_LOGGEDIN:
+				if (arg == 0) {
+					/* No auto-join channels */
+					GConfClient *client;
+					gboolean popup_channel_list;
+
+					client = gconf_client_get_default ();
+					popup_channel_list = gconf_client_get_bool (client,
+							"/apps/xchat/channel_list/auto_popup",  NULL);
+					g_object_unref (client);
+
+					if (popup_channel_list)
+						create_channel_list_window (sess, FALSE);
+				}
+				break;
+			case FE_SE_DISCONNECT:
+				navigation_model_set_disconnected (gui.tree_model, sess);
+				break;
 			}
 		}
 		list = list->next;
@@ -968,6 +1013,22 @@ fe_userlist_set_selected (struct session *sess)
 	/* FIXME: implement? */
 }
 
+void fe_flash_window (struct session *sess)
+{
+	if (!gtk_window_is_active (GTK_WINDOW (gui.main_window))) {
+		gtk_window_set_urgency_hint (GTK_WINDOW (gui.main_window), TRUE);
+	}
+}
+
+void fe_get_file (const char  *title,
+                  char        *initial,
+                  void       (*callback) (void *userdata, char *file),
+                  void        *userdata,
+                  int          flags)
+{
+	/* FIXME: implement */
+}
+
 static void
 setup_sm (gint argc, gchar *argv[])
 {
@@ -1023,14 +1084,55 @@ not_autoconnect (void)
 {
 	GSList *i;
 
-	if (arg_dont_autoconnect)
+	if (arg_dont_autoconnect) {
 		return TRUE;
+	}
 
 	for (i = network_list; i; i = g_slist_next (i)) {
 		ircnet *net = (ircnet *) (i->data);
-		if (net->flags & FLAG_AUTO_CONNECT)
+		if (net->flags & FLAG_AUTO_CONNECT) {
 			return FALSE;
+		}
 	}
 
 	return TRUE;
 }
+
+void
+fe_set_current (session *sess)
+{
+	// If find bar is open, hide it
+	find_bar_close (FIND_BAR (gui.find_bar));
+
+	gui.current_session = sess;
+
+	// Notify parts of the UI that the current session has changed
+	conversation_panel_set_current (CONVERSATION_PANEL (gui.conversation_panel), sess);
+	topic_label_set_current (TOPIC_LABEL (gui.topic_label), sess);
+	text_entry_set_current (TEXT_ENTRY (gui.text_entry), sess);
+	status_bar_set_current (STATUS_BAR (gui.status_bar), sess->server);
+	navigation_model_set_current (gui.tree_model, sess);
+
+	// Change the window name
+	if (sess->server->network == NULL) {
+		rename_main_window (NULL, sess->channel);
+	} else {
+		ircnet *net = sess->server->network;
+		rename_main_window (net->name, sess->channel);
+	}
+
+	// Set nickname button
+	set_nickname (sess->server, NULL);
+
+	// Set the label of the user list button
+	userlist_set_user_button (u, sess);
+	gtk_widget_set_sensitive (GTK_WIDGET (gui.userlist_toggle), sess->type == SESS_CHANNEL);
+
+	// FIXME: Userlist should be more encapsulated
+	gtk_tree_view_set_model (GTK_TREE_VIEW (gui.userlist), GTK_TREE_MODEL (userlist_get_store (u, sess)));
+
+	// Emit "focus tab" event for plugins that rely on it
+	plugin_emit_dummy_print (sess, "Focus Tab");
+
+	gtk_widget_grab_focus (GTK_WIDGET (gui.text_entry));
+}
