@@ -1,7 +1,7 @@
 /*
  * gui.c - main gui initialization and helper functions
  *
- * Copyright (C) 2004-2006 xchat-gnome team
+ * Copyright (C) 2004-2007 xchat-gnome team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -39,13 +39,14 @@ Userlist *u;
 gboolean
 initialize_gui_1 (void)
 {
-	gchar *path;
+	gui.initialized = FALSE;
 
 	gui.manager = gtk_ui_manager_new ();
 
-	path = locate_data_file ("xchat-gnome.glade");
+	gchar *path = locate_data_file ("xchat-gnome.glade");
 	gui.xml = glade_xml_new (path, NULL, NULL);
 	g_free (path);
+
 	if (!gui.xml) {
 		return FALSE;
 	}
@@ -63,8 +64,8 @@ initialize_gui_2 (void)
 	gui.tree_model = navigation_model_new ();
 	gui.server_tree = navigation_tree_new (gui.tree_model);
 	pixmaps_init ();
-	initialize_main_window ();
 	initialize_userlist ();
+	initialize_main_window ();
 
 	widget = glade_xml_get_widget (gui.xml, "server channel list");
 	gtk_widget_show (GTK_WIDGET (gui.server_tree));
@@ -72,6 +73,10 @@ initialize_gui_2 (void)
 
 	gui.dcc = dcc_window_new ();
 
+	gui.initialized = TRUE;
+
+	set_action_state (gui.server_tree);
+
 	return TRUE;
 }
 
