@@ -1,7 +1,7 @@
 /*
  * irc-network-editor.h - GtkDialog subclass for editing an IrcNetwork
  *
- * Copyright (C) 2004-2006 xchat-gnome team
+ * Copyright (C) 2004-2007 xchat-gnome team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -425,6 +425,9 @@ populate_autojoin_list (IrcNetworkEditor *e)
 	}
 
 	for (i = 0; channels[i]; i++) {
+		if (channels[i][0] == '\0') {
+			continue;
+		}
 		gtk_list_store_append (e->autojoin_store, &iter);
 		gtk_list_store_set (e->autojoin_store, &iter, 0, channels[i], -1);
 
