@@ -1,7 +1,7 @@
 /*
  * main-window.c - main GUI window functions
  *
- * Copyright (C) 2004-2006 xchat-gnome team
+ * Copyright (C) 2004-2007 xchat-gnome team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,7 +31,7 @@
 #include <fcntl.h>
 
 #include "about.h"
-#include "channel-list.h"
+#include "channel-list-window.h"
 #include "connect-dialog.h"
 #include "gui.h"
 #include "main-window.h"
@@ -51,7 +51,6 @@
 #include "../common/fe.h"
 
 static void on_main_window_close (GtkWidget *widget, GdkEvent *event, gpointer data);
-static void on_discussion_jump_activate (GtkAccelGroup *accelgroup, GObject *arg1, guint arg2, GdkModifierType arg3, gpointer data);
 static void on_pgup (GtkAccelGroup *accelgroup, GObject *arg1, guint arg2, GdkModifierType arg3, gpointer data);
 static void on_pgdn (GtkAccelGroup *accelgroup, GObject *arg1, guint arg2, GdkModifierType arg3, gpointer data);
 
@@ -65,19 +64,14 @@ static void on_edit_paste_activate (GtkAction *action, gpointer data);
 static void on_edit_preferences_activate (GtkAction *action, gpointer data);
 static void on_network_reconnect_activate (GtkAction *action, gpointer data);
 static void on_network_disconnect_activate (GtkAction *action, gpointer data);
-static void on_network_close_activate (GtkAction *action, gpointer data);
 static void on_network_channels_activate (GtkAction *action, gpointer data);
 static void on_discussion_save_activate (GtkAction *action, gpointer data);
 static void on_discussion_leave_activate (GtkAction *action, gpointer data);
-static void on_discussion_close_activate (GtkAction *action, gpointer data);
+static void on_close_activate (GtkAction *action, gpointer data);
 static void on_discussion_find_activate (GtkAction *action, gpointer data);
 static void on_discussion_bans_activate (GtkAction *action, gpointer data);
 static void on_discussion_topic_change_activate (GtkButton *widget, gpointer data);
 static void on_discussion_users_activate (GtkAction *action, gpointer data);
-static void on_go_previous_network_activate (GtkAction *action, gpointer data);
-static void on_go_next_network_activate (GtkAction *action, gpointer data);
-static void on_go_previous_discussion_activate (GtkAction *action, gpointer data);
-static void on_go_next_discussion_activate (GtkAction *action, gpointer data);
 static void on_help_contents_activate (GtkAction *action, gpointer data);
 static void on_help_about_activate (GtkAction *action, gpointer data);
 static void on_nickname_clicked (GtkButton *widget, gpointer user_data);
@@ -91,6 +85,8 @@ static gboolean on_hpane_move (GtkPaned *widget, GParamSpec *param_spec, gpointe
 static gboolean on_main_window_focus_in (GtkWidget *widget, GdkEventFocus *event, gpointer data);
 static gboolean on_main_window_configure (GtkWidget *widget, GdkEventConfigure *event, gpointer data);
 
+static void nickname_style_set (GtkWidget *button, GtkStyle *previous_style, gpointer data);
+
 static GtkActionEntry action_entries [] = {
 
 	/* Toplevel */
@@ -104,7 +100,7 @@ static GtkActionEntry action_entries [] = {
 	{ "PopupAction", NULL, "" },
 
 	/* IRC menu */
-	{ "IRCConnect",   NULL,           N_("_Connect"),        "<control>N", NULL, G_CALLBACK (on_irc_connect_activate) },
+	{ "IRCConnect",   NULL,           N_("_Connect..."),        "<control>N", NULL, G_CALLBACK (on_irc_connect_activate) },
 	{ "IRCDownloads", NULL,           N_("_File Transfers"), "<alt>F",     NULL, G_CALLBACK (on_irc_downloads_activate) },
 	{ "IRCQuit",      GTK_STOCK_QUIT, N_("_Quit"),           "<control>Q", NULL, G_CALLBACK (on_irc_quit_activate) },
 
@@ -117,27 +113,21 @@ static GtkActionEntry action_entries [] = {
 	/* Network menu */
 	{ "NetworkReconnect",   GTK_STOCK_REFRESH,     N_("_Reconnect"),   "<control>R",        NULL, G_CALLBACK (on_network_reconnect_activate) },
 	{ "NetworkDisconnect",  GTK_STOCK_DISCONNECT,  N_("_Disconnect"),  "",                  NULL, G_CALLBACK (on_network_disconnect_activate) },
-	{ "NetworkClose",       GTK_STOCK_CLOSE,       N_("_Close"),       "<shift><control>W", NULL, G_CALLBACK (on_network_close_activate) },
+	{ "NetworkClose",       GTK_STOCK_CLOSE,       N_("_Close"),       "<shift><control>W", NULL, G_CALLBACK (on_close_activate) },
 	{ "NetworkChannels",    NULL,                  N_("_Channels..."), "<alt>C",            NULL, G_CALLBACK (on_network_channels_activate) },
 
 	/* Discussion menu */
 	{ "DiscussionSave",        GTK_STOCK_SAVE,           N_("_Save Transcript"), "<control>S", NULL, G_CALLBACK (on_discussion_save_activate) },
 	{ "DiscussionLeave",       GTK_STOCK_QUIT,           N_("_Leave"),           "",           NULL, G_CALLBACK (on_discussion_leave_activate) },
-	{ "DiscussionClose",       GTK_STOCK_CLOSE,          N_("Cl_ose"),           "<control>W", NULL, G_CALLBACK (on_discussion_close_activate) },
+	{ "DiscussionClose",       GTK_STOCK_CLOSE,          N_("Cl_ose"),           "<control>W", NULL, G_CALLBACK (on_close_activate) },
 	{ "DiscussionFind",        GTK_STOCK_FIND,           N_("_Find"),            "<control>F", NULL, G_CALLBACK (on_discussion_find_activate) },
 	{ "DiscussionChangeTopic", NULL,                     N_("Change _Topic"),    "<alt>T",     NULL, G_CALLBACK (on_discussion_topic_change_activate) },
 	{ "DiscussionBans",        GTK_STOCK_DIALOG_WARNING, N_("_Bans..."),         "<alt>B",     NULL, G_CALLBACK (on_discussion_bans_activate) },
 	{ "DiscussionUsers",       NULL,                     N_("_Users"),           "<control>U", NULL, G_CALLBACK (on_discussion_users_activate) },
 
-	/* Go menu */
-	{ "GoPreviousNetwork",    NULL, N_("Pre_vious Network"),    "<control>Up",   NULL, G_CALLBACK (on_go_previous_network_activate) },
-	{ "GoNextNetwork",        NULL, N_("Nex_t Network"),        "<control>Down", NULL, G_CALLBACK (on_go_next_network_activate) },
-	{ "GoPreviousDiscussion", NULL, N_("_Previous Discussion"), "<alt>Up",       NULL, G_CALLBACK (on_go_previous_discussion_activate) },
-	{ "GoNextDiscussion",     NULL, N_("_Next Discussion"),     "<alt>Down",     NULL, G_CALLBACK (on_go_next_discussion_activate) },
-
 	/* Help menu */
-	{ "HelpContents", GTK_STOCK_HELP,  N_("_Contents"), "", NULL, G_CALLBACK (on_help_contents_activate) },
-	{ "HelpAbout",    GTK_STOCK_ABOUT, N_("_About"),    "", NULL, G_CALLBACK (on_help_about_activate) },
+	{ "HelpContents", GTK_STOCK_HELP,  N_("_Contents"), "F1", NULL, G_CALLBACK (on_help_contents_activate) },
+	{ "HelpAbout",    GTK_STOCK_ABOUT, N_("_About"),    NULL, NULL, G_CALLBACK (on_help_about_activate) },
 };
 
 void
@@ -145,14 +135,13 @@ initialize_main_window (void)
 {
 	GtkWidget *close, *menu_vbox, *widget;
 	GtkSizeGroup *group;
+	GtkAction *action;
+	gchar *path;
 
 	gui.main_window = glade_xml_get_widget (gui.xml, "xchat-gnome");
-	g_signal_connect (G_OBJECT (gui.main_window), "delete-event",
-	                  G_CALLBACK (on_main_window_close), NULL);
-	g_signal_connect (G_OBJECT (gui.main_window), "focus-in-event",
-	                  G_CALLBACK (on_main_window_focus_in), NULL);
-	g_signal_connect (G_OBJECT (gui.main_window), "configure-event",
-	                  G_CALLBACK (on_main_window_configure), NULL);
+	g_signal_connect (G_OBJECT (gui.main_window), "delete-event",    G_CALLBACK (on_main_window_close),     NULL);
+	g_signal_connect (G_OBJECT (gui.main_window), "focus-in-event",  G_CALLBACK (on_main_window_focus_in),  NULL);
+	g_signal_connect (G_OBJECT (gui.main_window), "configure-event", G_CALLBACK (on_main_window_configure), NULL);
 
 	/* hook up the menus */
 	gui.action_group = gtk_action_group_new ("MenuAction");
@@ -166,23 +155,26 @@ initialize_main_window (void)
 	g_signal_connect (gui.manager, "add-widget", G_CALLBACK (on_add_widget), menu_vbox);
 
 	/* load the menus */
-	if (g_file_test ("../../data/xchat-gnome-ui.xml", G_FILE_TEST_EXISTS))
-		gtk_ui_manager_add_ui_from_file (gui.manager, "../../data/xchat-gnome-ui.xml", NULL);
-	else
-		gtk_ui_manager_add_ui_from_file (gui.manager, XCHATSHAREDIR "/xchat-gnome-ui.xml", NULL);
+	path = locate_data_file ("xchat-gnome-ui.xml");
+	gtk_ui_manager_add_ui_from_file (gui.manager, path, NULL);
+	g_free (path);
 
 	/* hook up accelerators */
 	gtk_window_add_accel_group (GTK_WINDOW (gui.main_window), gtk_ui_manager_get_accel_group (gui.manager));
 
 	close = glade_xml_get_widget (gui.xml, "close discussion");
-	g_signal_connect (G_OBJECT (close), "clicked", G_CALLBACK (on_discussion_close_activate), NULL);
+	action = gtk_action_group_get_action (gui.action_group, "DiscussionClose");
+	gtk_action_connect_proxy (action, close);
 
-	gui.conversation_panel = glade_xml_get_widget (gui.xml, "conversation_panel");
-	gui.find_bar           = glade_xml_get_widget (gui.xml, "find_bar");
-	gui.status_bar         = glade_xml_get_widget (gui.xml, "status_bar");
-	gui.text_entry         = glade_xml_get_widget (gui.xml, "text_entry");
-	gui.topic_hbox         = glade_xml_get_widget (gui.xml, "topic_hbox");
-	gui.topic_label        = glade_xml_get_widget (gui.xml, "topic_label");
+#define GW(name) ((gui.name) = glade_xml_get_widget (gui.xml, #name))
+	GW(conversation_panel);
+	GW(find_bar);
+	GW(status_bar);
+	GW(text_entry);
+	GW(topic_hbox);
+	GW(topic_label);
+	GW(nick_button);
+#undef GW
 
 	/* Hook up accelerators for pgup/pgdn */
 	{
@@ -205,95 +197,29 @@ initialize_main_window (void)
 		gtk_window_add_accel_group (GTK_WINDOW (gui.main_window), pg_accel);
 	}
 
-	/* Hook up accelerators for alt-#. */
-	{
-		GtkAccelGroup *discussion_accel;
-		GClosure *closure;
-		int i;
-		gchar num[2] = {0,0}; /* Will be used to help determine the keyval. */
-
-		/* Create our accelerator group. */
-		discussion_accel = gtk_accel_group_new ();
-
-		/* For alt-1 through alt-9 we just loop to set up the accelerators.
-		 * We want the data passed with the callback to be one less then the
-		 * button pressed (e.g. alt-1 requests the channel who's path is 0:0)
-		 * so we loop from 0 <= i < 1. We use i for the user data and the ascii
-		 * value of i+1 to get the keyval.
-		 */
-		for (i = 0; i < 9; i++) {
-			/* num is a string containing the ascii value of i+1. */
-			num[0] = i + '1';
-
-			/* Set up our GClosure with user data set to i. */
-			closure = g_cclosure_new (G_CALLBACK (on_discussion_jump_activate), GINT_TO_POINTER (i), NULL);
-
-			/* Connect up the accelerator. */
-			gtk_accel_group_connect (discussion_accel, gdk_keyval_from_name (num), GDK_MOD1_MASK, GTK_ACCEL_VISIBLE, closure);
-
-			/* Delete the reference to the GClosure. */
-			g_closure_unref (closure);
-		}
-
-		/* Now we set up keypress alt-0 with user data 9. */
-		closure = g_cclosure_new (G_CALLBACK (on_discussion_jump_activate), GUINT_TO_POINTER (9), NULL);
-		gtk_accel_group_connect (discussion_accel, gdk_keyval_from_name ("0"), GDK_MOD1_MASK, GTK_ACCEL_VISIBLE, closure);
-		g_closure_unref (closure);
-
-		/* alt-+ */
-		closure = g_cclosure_new (G_CALLBACK (on_go_next_discussion_activate), NULL, NULL);
-		gtk_accel_group_connect (discussion_accel, gdk_keyval_from_name ("equal"), GDK_MOD1_MASK, GTK_ACCEL_VISIBLE, closure);
-		g_closure_unref (closure);
-
-		/* alt-- */
-		closure = g_cclosure_new (G_CALLBACK (on_go_previous_discussion_activate), NULL, NULL);
-		gtk_accel_group_connect (discussion_accel, gdk_keyval_from_name ("minus"), GDK_MOD1_MASK, GTK_ACCEL_VISIBLE, closure);
-		g_closure_unref (closure);
-
-		/* We've had a couple of calls to hook up Ctrl-Alt-PgUp and
-		 * Ctrl-Alt-PgDown to discussion navigation. As far as I can
-		 * tell this is not HIG compliant, but for the time being we'll
-		 * put it in. It's easy enough to delete it later.
-		 */
-		closure = g_cclosure_new (G_CALLBACK (on_go_next_discussion_activate), NULL, NULL);
-		gtk_accel_group_connect (discussion_accel, GDK_Page_Down,
-				GDK_MOD1_MASK | GDK_CONTROL_MASK , GTK_ACCEL_VISIBLE, closure);
-
-		g_closure_unref (closure);
-
-		closure = g_cclosure_new (G_CALLBACK (on_go_previous_discussion_activate), NULL, NULL);
-		gtk_accel_group_connect (discussion_accel, GDK_Page_Up,
-				GDK_MOD1_MASK | GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE, closure);
-
-		g_closure_unref (closure);
-
-		/* Add the accelgroup to the main window. */
-		gtk_window_add_accel_group (GTK_WINDOW (gui.main_window), discussion_accel);
-	}
+	navigation_tree_add_accels (gui.server_tree, GTK_WINDOW (gui.main_window));
 
 	/* Size group between users button and entry field */
 	group = gtk_size_group_new (GTK_SIZE_GROUP_VERTICAL);
 	gui.userlist_toggle = glade_xml_get_widget (gui.xml, "userlist_toggle");
 	g_signal_connect (G_OBJECT (gui.userlist_toggle), "toggled", G_CALLBACK (on_users_toggled), NULL);
-	if (g_file_test ("../../data/users.png", G_FILE_TEST_EXISTS))
-		gtk_button_set_image (GTK_BUTTON (gui.userlist_toggle), gtk_image_new_from_file ("../../data/users.png"));
-	else
-		gtk_button_set_image (GTK_BUTTON (gui.userlist_toggle), gtk_image_new_from_file (XCHATSHAREDIR "/users.png"));
+
+	gtk_button_set_image (GTK_BUTTON (gui.userlist_toggle), gtk_image_new_from_icon_name ("xchat-gnome-users", GTK_ICON_SIZE_MENU));
 	gtk_size_group_add_widget (group, gui.userlist_toggle);
 	widget = glade_xml_get_widget (gui.xml, "entry hbox");
 	gtk_size_group_add_widget (group, widget);
 	g_object_unref (group);
 
 	/* connect nickname button */
-	widget = glade_xml_get_widget (gui.xml, "nickname");
-	gtk_button_set_use_underline (GTK_BUTTON (widget), FALSE);
-	g_signal_connect (G_OBJECT (widget), "clicked", G_CALLBACK (on_nickname_clicked), NULL);
+	gtk_button_set_use_underline (GTK_BUTTON (gui.nick_button), FALSE);
+	g_signal_connect (G_OBJECT (gui.nick_button), "clicked",   G_CALLBACK (on_nickname_clicked), NULL);
+	g_signal_connect (G_OBJECT (GTK_BIN (gui.nick_button)->child), "style-set", G_CALLBACK (nickname_style_set),  NULL);
 }
 
 void
 run_main_window ()
 {
-	GtkWidget *pane, *widget;
+	GtkWidget *pane;
 	GConfClient *client;
 	int width, height;
 	int x, y;
@@ -302,14 +228,16 @@ run_main_window ()
 	client = gconf_client_get_default ();
 	width  = gconf_client_get_int (client, "/apps/xchat/main_window/width",  NULL);
 	height = gconf_client_get_int (client, "/apps/xchat/main_window/height", NULL);
-	if(width == 0 || height == 0)
+	if (width == 0 || height == 0) {
 		gtk_window_set_default_size (GTK_WINDOW (gui.main_window), 800, 550);
-	else
+	} else {
 		gtk_window_set_default_size (GTK_WINDOW (gui.main_window), width, height);
+	}
 	x = gconf_client_get_int (client, "/apps/xchat/main_window/x", NULL);
 	y = gconf_client_get_int (client, "/apps/xchat/main_window/y", NULL);
-	if (!(x == 0 || y == 0))
+	if (!(x == 0 || y == 0)) {
 		gtk_window_move (GTK_WINDOW (gui.main_window), x, y);
+	}
 	h = gconf_client_get_int (client, "/apps/xchat/main_window/hpane", NULL);
 	if(h != 0) {
 		GtkWidget *hpane = glade_xml_get_widget (gui.xml, "HPane");
@@ -320,12 +248,6 @@ run_main_window ()
 	g_signal_connect (G_OBJECT (pane), "notify::position", G_CALLBACK (on_hpane_move), NULL);
 	g_object_unref (client);
 
-	/* Temporarily disable menu items */
-	widget = gtk_ui_manager_get_widget (gui.manager, "/ui/menubar/DiscussionMenu/DiscussionBansItem");
-	gtk_widget_set_sensitive (widget, FALSE);
-	widget = gtk_ui_manager_get_widget (gui.manager, "/ui/ChannelJoinedPopup/ChannelBans");
-	gtk_widget_set_sensitive (widget, FALSE);
-
 	gtk_widget_show (gui.main_window);
 }
 
@@ -335,10 +257,11 @@ rename_main_window (gchar *server, gchar *channel)
 	gchar *new_title;
 
 	if (server == NULL) {
-		if (channel && strlen (channel) != 0)
+		if (channel && strlen (channel) != 0) {
 			gtk_window_set_title (GTK_WINDOW (gui.main_window), channel);
-		else
+		} else {
 			gtk_window_set_title (GTK_WINDOW (gui.main_window), "XChat-GNOME");
+		}
 		return;
 	}
 	new_title = g_strconcat (server, ": ", channel, NULL);
@@ -403,12 +326,13 @@ on_edit_cut_activate (GtkAction *action, gpointer data)
 static void
 on_edit_copy_activate (GtkAction *action, gpointer data)
 {
-	if (gtk_editable_get_selection_bounds (GTK_EDITABLE (gui.text_entry), NULL, NULL))
+	if (gtk_editable_get_selection_bounds (GTK_EDITABLE (gui.text_entry), NULL, NULL)) {
 		/* There is something selected in the text_entry */
 		gtk_editable_copy_clipboard (GTK_EDITABLE (gui.text_entry));
-	else
+	} else {
 		/* Nothing selected, we copy from the conversation panel */
 		conversation_panel_copy_selection (CONVERSATION_PANEL (gui.conversation_panel));
+	}
 }
 
 static void
@@ -423,7 +347,7 @@ on_edit_preferences_activate (GtkAction *action, gpointer data)
 	if (!gui.prefs_dialog) {
 		gui.prefs_dialog = preferences_dialog_new ();
 		g_object_add_weak_pointer (G_OBJECT (gui.prefs_dialog),
-					   (gpointer *) (&gui.prefs_dialog));
+		                           (gpointer *) (&gui.prefs_dialog));
 	}
 
 	preferences_dialog_show (gui.prefs_dialog);
@@ -439,29 +363,9 @@ static void
 on_network_disconnect_activate (GtkAction *action, gpointer data)
 {
 	session *s = gui.current_session;
-	if (s)
+	if (s) {
 		s->server->disconnect (s, TRUE, -1);
-}
-
-static void
-on_network_close_activate (GtkAction *actoin, gpointer data)
-{
-	GtkTreeIter parent, *iter;
-	session *sess = gui.current_session;
-	if (sess == NULL)
-		return;
-
-	iter = navigation_model_get_unsorted_iter (gui.tree_model, sess);
-
-	sess->server->disconnect (sess, TRUE, -1);
-
-	if (gtk_tree_model_iter_parent (GTK_TREE_MODEL (gui.tree_model->store), &parent, iter)) {
-		gtk_tree_store_remove (gui.tree_model->store, &parent);
-	} else {
-		gtk_tree_store_remove (gui.tree_model->store, iter);
 	}
-
-	gtk_tree_iter_free (iter);
 }
 
 static void
@@ -473,7 +377,7 @@ on_irc_downloads_activate (GtkAction *action, gpointer data)
 static void
 on_network_channels_activate (GtkAction *action, gpointer data)
 {
-	create_channel_list (gui.current_session);
+	create_channel_list_window (gui.current_session, TRUE);
 }
 
 static void
@@ -498,8 +402,9 @@ on_discussion_leave_activate (GtkAction *action, gpointer data)
 
 		client = gconf_client_get_default ();
 		text = gconf_client_get_string (client, "/apps/xchat/irc/partmsg", NULL);
-		if (text == NULL)
+		if (text == NULL) {
 			text = g_strdup (_("Ex-Chat"));
+		}
 		s->server->p_part (s->server, s->channel, text);
 		g_object_unref (client);
 		g_free (text);
@@ -507,16 +412,36 @@ on_discussion_leave_activate (GtkAction *action, gpointer data)
 }
 
 static void
-on_discussion_close_activate (GtkAction *action, gpointer data)
+on_close_activate (GtkAction *action, gpointer data)
 {
 	session *s = gui.current_session;
-	if (s == NULL)
+	if (s == NULL) {
 		return;
-/*	navigation_tree_select_next_channel (gui.server_tree, TRUE);*/
+	}
+
+	switch (s->type) {
+	case SESS_CHANNEL:
+	{
+		GConfClient *client = gconf_client_get_default ();
+		gchar *text = gconf_client_get_string (client, "/apps/xchat/irc/partmsg", NULL);
+		if (text == NULL) {
+			text = g_strdup (_("Ex-Chat"));
+		}
+
+		s->server->p_part (s->server, s->channel, text);
+
+		g_object_unref (client);
+		g_free (text);
+
+		break;
+	}
+
+	case SESS_SERVER:
+		s->server->disconnect (s, TRUE, -1);
+		break;
+	}
+
 	fe_close_window (s);
-	conversation_panel_remove_session (CONVERSATION_PANEL (gui.conversation_panel), s);
-	topic_label_remove_session        (TOPIC_LABEL        (gui.topic_label),        s);
-	text_entry_remove_session         (TEXT_ENTRY         (gui.text_entry),         s);
 }
 
 static void
@@ -531,36 +456,6 @@ on_discussion_bans_activate (GtkAction *action, gpointer data)
 	/* FIXME: implement */
 }
 
-static void
-on_go_previous_network_activate (GtkAction *action, gpointer data)
-{
-	navigation_tree_select_prev_network (gui.server_tree);
-}
-
-static void
-on_go_next_network_activate(GtkAction *action, gpointer data)
-{
-	navigation_tree_select_next_network (gui.server_tree);
-}
-
-static void
-on_go_previous_discussion_activate (GtkAction *action, gpointer data)
-{
-	navigation_tree_select_prev_channel (gui.server_tree);
-}
-
-static void
-on_go_next_discussion_activate (GtkAction *action, gpointer data)
-{
-	navigation_tree_select_next_channel (gui.server_tree);
-}
-
-static void
-on_discussion_jump_activate (GtkAccelGroup *accelgroup, GObject *arg1, guint arg2, GdkModifierType arg3, gpointer data)
-{
-	navigation_tree_select_nth_channel (gui.server_tree, GPOINTER_TO_INT (data));
-}
-
 static void
 on_pgup (GtkAccelGroup *accelgroup, GObject *arg1, guint arg2, GdkModifierType arg3, gpointer data)
 {
@@ -604,8 +499,9 @@ on_nickname_clicked (GtkButton *widget, gpointer user_data)
 	GtkWidget *entry, *away;
 	gint result;
 
-	if (gui.current_session == NULL)
+	if (gui.current_session == NULL) {
 		return;
+	}
 	current_sess = gui.current_session;
 
 	dialog = glade_xml_get_widget (gui.xml, "nickname dialog");
@@ -627,22 +523,24 @@ on_nickname_clicked (GtkButton *widget, gpointer user_data)
 
 		/* Nick */
 		text = (gchar *) gtk_entry_get_text (GTK_ENTRY (entry));
-		if (all)
+		if (all) {
 			buf = g_strdup_printf ("allserv nick %s", text);
-		else
+		} else {
 			buf = g_strdup_printf ("nick %s", text);
+		}
 		handle_command (current_sess, buf, FALSE);
 		g_free (buf);
 
 		/* Away */
 		if (current_sess->server->is_away != gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (away))) {
-			if (all)
+			if (all) {
 				handle_command (current_sess, "allserv away", FALSE);
-			else
+			} else {
 				handle_command (current_sess, "away", FALSE);	
+			}
 		}
 	}
-	gtk_widget_hide (dialog);	
+	gtk_widget_hide (dialog);
 }
 
 static gboolean
@@ -683,24 +581,35 @@ on_users_toggled (GtkToggleButton *widget, gpointer user_data)
 
 	toggled = gtk_toggle_button_get_active (widget);
 
-	if (toggled)
+	if (toggled) {
 		userlist_gui_show ();
-	else
+	} else {
 		userlist_gui_hide ();
+	}
 }
 
 void
 set_nickname (struct server *serv, char *newnick)
 {
-	if (gui.current_session == NULL)
+	GtkLabel *label;
+
+	if (gui.current_session == NULL) {
 		return;
+	}
+
+	label = GTK_LABEL (GTK_BIN (gui.nick_button)->child);
+
+	if (serv == NULL) {
+		gtk_label_set_text (label, "");
+		return;
+	}
 
 	if (serv == gui.current_session->server) {
-		GtkWidget *nick = glade_xml_get_widget (gui.xml, "nickname");
-		if (newnick == NULL)
-			gtk_button_set_label (GTK_BUTTON (nick), serv->nick);
-		else
-			gtk_button_set_label (GTK_BUTTON (nick), newnick);
+		if (newnick == NULL) {
+			gtk_label_set_text (label, serv->nick);
+		} else {
+			gtk_label_set_text (label, newnick);
+		}
 		set_nickname_color (serv);
 	}
 }
@@ -708,24 +617,26 @@ set_nickname (struct server *serv, char *newnick)
 void
 set_nickname_color (struct server *serv)
 {
-	if (gui.current_session == NULL)
+	if (gui.current_session == NULL) {
 		return;
+	}
 
 	if (serv == gui.current_session->server) {
-		GtkWidget *nick_button;
 		GtkLabel *label;
 		PangoAttribute *attr;
 		PangoAttrList *l;
+		GtkStyle *style;
 		GdkColor *color;
 
 		l = pango_attr_list_new ();
-		nick_button = glade_xml_get_widget (gui.xml, "nickname");
-		label = GTK_LABEL (GTK_BIN (nick_button)->child);
+		label = GTK_LABEL (GTK_BIN (gui.nick_button)->child);
+
+		style = gtk_widget_get_style (GTK_WIDGET (label));
 
 		if (serv->is_away) {
-			color = &(nick_button->style->fg[GTK_STATE_INSENSITIVE]);
+			color = &(style->fg[GTK_STATE_INSENSITIVE]);
 		} else {
-			color = &(nick_button->style->fg[GTK_STATE_NORMAL]);
+			color = &(style->fg[GTK_STATE_NORMAL]);
 		}
 		attr = pango_attr_foreground_new (color->red, color->green, color->blue);
 
@@ -752,3 +663,13 @@ on_main_window_configure (GtkWidget *widget, GdkEventConfigure *event, gpointer
 	conversation_panel_queue_tdraw (CONVERSATION_PANEL (gui.conversation_panel));
 	return FALSE;
 }
+
+static void
+nickname_style_set (GtkWidget *button, GtkStyle *previous_style, gpointer data)
+{
+	if (gui.current_session == NULL) {
+		return;
+	}
+
+	set_nickname_color (gui.current_session->server);
+}
