@@ -1,7 +1,7 @@
 /*
  * preferences-page-colors.c - helpers for the colors preferences page
  *
- * Copyright (C) 2004-2006 xchat-gnome team
+ * Copyright (C) 2004-2007 xchat-gnome team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,17 +28,33 @@
 #include "preferences-page-colors.h"
 #include "preferences-dialog.h"
 #include "xtext.h"
+#include "util.h"
+
+G_DEFINE_TYPE(PreferencesPageColors, preferences_page_colors, PREFERENCES_PAGE_TYPE)
 
 static int scheme;
 
+static void
+show_colors_changed (GtkToggleButton *button, const gchar *key)
+{
+	GConfClient *client;
+	gboolean value;
+
+	client = gconf_client_get_default ();
+	value = gtk_toggle_button_get_active (button);
+	gconf_client_set_bool (client, key, value, NULL);
+	g_object_unref (client);
+}
+
 static void
 color_button_changed (GtkColorButton *button, gpointer data)
 {
 	int index = GPOINTER_TO_INT (data);
 	GdkColor c;
 
-	if (scheme != 2)
+	if (scheme != 2) {
 		return;
+	}
 
 	gtk_color_button_get_color (button, &c);
 	if (index < 32) {
@@ -96,7 +112,7 @@ set_palette_buttons (int selection, GtkWidget **palette_buttons)
 }
 
 static void
-colors_changed (GtkComboBox *combo_box, PreferencesColorsPage *page)
+colors_changed (GtkComboBox *combo_box, PreferencesPageColors *page)
 {
 	int i, selection;
 	GConfClient *client;
@@ -110,10 +126,12 @@ colors_changed (GtkComboBox *combo_box, PreferencesColorsPage *page)
 	sensitive = (selection == 2);
 
 	/* If we've set custom, sensitize the color buttons */
-	for (i = 0; i < 4; i++)
+	for (i = 0; i < 4; i++) {
 		gtk_widget_set_sensitive (page->color_buttons[i], sensitive);
-	for (i = 0; i < 32; i++)
+	}
+	for (i = 0; i < 32; i++) {
 		gtk_widget_set_sensitive (page->palette_buttons[i], sensitive);
+	}
 
 	gtk_widget_set_sensitive (page->mirc_colors_box, sensitive);
 	gtk_widget_set_sensitive (page->extra_colors_box, sensitive);
@@ -126,7 +144,18 @@ colors_changed (GtkComboBox *combo_box, PreferencesColorsPage *page)
 }
 
 static void
-gconf_color_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, PreferencesColorsPage *page)
+gconf_show_colors_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, GtkToggleButton *button)
+{
+	gboolean toggle;
+
+	g_signal_handlers_block_by_func (button, "toggled", show_colors_changed);
+	toggle = gconf_client_get_bool (client, entry->key, NULL);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), toggle);
+	g_signal_handlers_unblock_by_func (button, "toggled", show_colors_changed);
+}
+
+static void
+gconf_color_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, PreferencesPageColors *page)
 {
 	int selection;
 	selection = gconf_client_get_int (client, entry->key, NULL);
@@ -137,18 +166,20 @@ gconf_color_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, Pref
 	set_palette_buttons (selection, page->palette_buttons);
 }
 
-PreferencesColorsPage *
+PreferencesPageColors *
 preferences_page_colors_new (gpointer prefs_dialog, GladeXML *xml)
 {
-	PreferencesColorsPage *page = g_new0 (PreferencesColorsPage, 1);
+	PreferencesPageColors *page = g_object_new (PREFERENCES_PAGE_COLORS_TYPE, NULL);
 	PreferencesDialog *p = (PreferencesDialog *) prefs_dialog;
 	GtkSizeGroup *group;
 	GtkTreeIter iter;
 	gint i, j;
+	gboolean toggle;
 
 	palette_init ();
 
 #define GW(name) ((page->name) = glade_xml_get_widget (xml, #name))
+	GW(show_colors);
 	GW(color_label_1);
 	GW(color_label_2);
 	GW(color_label_3);
@@ -210,17 +241,16 @@ preferences_page_colors_new (gpointer prefs_dialog, GladeXML *xml)
 	gtk_label_set_mnemonic_widget (GTK_LABEL (page->color_label_4), page->color_buttons[2]);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (page->color_label_5), page->color_buttons[3]);
 
+	g_signal_connect (G_OBJECT (page->show_colors),      "toggled",   G_CALLBACK (show_colors_changed),  "/apps/xchat/irc/showcolors");
 	g_signal_connect (G_OBJECT (page->color_buttons[0]), "color-set", G_CALLBACK (color_button_changed), GINT_TO_POINTER (32));
 	g_signal_connect (G_OBJECT (page->color_buttons[1]), "color-set", G_CALLBACK (color_button_changed), GINT_TO_POINTER (33));
 	g_signal_connect (G_OBJECT (page->color_buttons[2]), "color-set", G_CALLBACK (color_button_changed), GINT_TO_POINTER (34));
 	g_signal_connect (G_OBJECT (page->color_buttons[3]), "color-set", G_CALLBACK (color_button_changed), GINT_TO_POINTER (35));
 
-	if (g_file_test ("../../data/color.png", G_FILE_TEST_EXISTS))
-		page->icon = gdk_pixbuf_new_from_file ("../../data/color.png", NULL);
-	else
-		page->icon = gdk_pixbuf_new_from_file (XCHATSHAREDIR "/color.png", NULL);
+	PREFERENCES_PAGE (page)->icon = gtk_widget_render_icon (page->show_colors, GTK_STOCK_SELECT_COLOR, GTK_ICON_SIZE_MENU, NULL);
+
 	gtk_list_store_append (p->page_store, &iter);
-	gtk_list_store_set (p->page_store, &iter, 0, page->icon, 1, _("Colors"), 2, 1, -1);
+	gtk_list_store_set (p->page_store, &iter, 0, PREFERENCES_PAGE (page)->icon, 1, _("Colors"), 2, 1, -1);
 
 	page->combo = gtk_combo_box_new_text ();
 	gtk_combo_box_append_text (GTK_COMBO_BOX (page->combo), _("Black on White"));
@@ -232,8 +262,13 @@ preferences_page_colors_new (gpointer prefs_dialog, GladeXML *xml)
 	gtk_box_pack_start (GTK_BOX (page->foreground_background_hbox), page->combo, FALSE, TRUE, 0);
 	scheme = gconf_client_get_int (p->gconf, "/apps/xchat/irc/color_scheme", NULL);
 
-	page->notify = gconf_client_notify_add (p->gconf, "/apps/xchat/irc/color_scheme",
-	                                        (GConfClientNotifyFunc) gconf_color_changed, page, NULL, NULL);
+	page->nh[0] = gconf_client_notify_add (p->gconf, "/apps/xchat/irc/showcolors",
+	                                       (GConfClientNotifyFunc) gconf_show_colors_changed, page->show_colors, NULL, NULL);
+	page->nh[1] = gconf_client_notify_add (p->gconf, "/apps/xchat/irc/color_scheme",
+	                                       (GConfClientNotifyFunc) gconf_color_changed, page, NULL, NULL);
+
+	toggle = gconf_client_get_bool (p->gconf, "/apps/xchat/irc/showcolors", NULL);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (page->show_colors), toggle);
 
 	g_signal_connect (G_OBJECT (page->combo), "changed", G_CALLBACK (colors_changed), page);
 	gtk_combo_box_set_active (GTK_COMBO_BOX (page->combo), scheme);
@@ -241,14 +276,40 @@ preferences_page_colors_new (gpointer prefs_dialog, GladeXML *xml)
 	return page;
 }
 
-void
-preferences_page_colors_free (PreferencesColorsPage *page)
+static void
+preferences_page_colors_init (PreferencesPageColors *page)
 {
+}
+
+static void
+preferences_page_colors_dispose (GObject *object)
+{
+	//PreferencesPageColors *page = (PreferencesPageColors *) object;
+
+	G_OBJECT_CLASS (preferences_page_colors_parent_class)->dispose (object);
+}	
+
+static void
+preferences_page_colors_finalize (GObject *object)
+{
+	PreferencesPageColors *page = (PreferencesPageColors *) object;
+	gint i;
 	GConfClient *client;
 
 	client = gconf_client_get_default ();
-	gconf_client_notify_remove (client, page->notify);
+	for (i = 0; i < 2; i++) {
+		gconf_client_notify_remove (client, page->nh[i]);
+	}
 	g_object_unref (client);
-	g_object_unref (page->icon);
-	g_free (page);
+
+	G_OBJECT_CLASS (preferences_page_colors_parent_class)->finalize (object);
+}
+
+static void
+preferences_page_colors_class_init (PreferencesPageColorsClass *klass)
+{
+	GObjectClass *object_class = (GObjectClass *) klass;
+
+	object_class->dispose = preferences_page_colors_dispose;
+	object_class->finalize = preferences_page_colors_finalize;
 }
