@@ -1,7 +1,7 @@
 /*
  * preferences-page-dcc.c - helpers for the DCC preferences page
  *
- * Copyright (C) 2004-2006 xchat-gnome team
+ * Copyright (C) 2004-2007 xchat-gnome team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,9 +24,12 @@
 #include <string.h>
 #include "preferences-page-dcc.h"
 #include "preferences-dialog.h"
+#include "util.h"
 #include "../common/xchat.h"
 #include "../common/xchatc.h"
 
+G_DEFINE_TYPE(PreferencesPageDCC, preferences_page_dcc, PREFERENCES_PAGE_TYPE)
+
 static void
 download_dir_changed (GtkFileChooser *file_chooser, gpointer data)
 {
@@ -74,7 +77,7 @@ autoaccept_file_changed (GtkToggleButton *button, gpointer data)
 }
 
 static void
-get_ip_from_server_changed (GtkRadioButton *button, PreferencesDCCPage *page)
+get_ip_from_server_changed (GtkRadioButton *button, PreferencesPageDCC *page)
 {
 	gboolean on = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
 	gtk_widget_set_sensitive (page->special_ip_address, !on);
@@ -113,11 +116,11 @@ grt_changed (GtkSpinButton *button, gpointer data)
 	prefs.dcc_global_max_get_cps = gtk_spin_button_get_value_as_int (button);
 }
 
-PreferencesDCCPage *
+PreferencesPageDCC *
 preferences_page_dcc_new (gpointer prefs_dialog, GladeXML *xml)
 {
+	PreferencesPageDCC *page = g_object_new (PREFERENCES_PAGE_DCC_TYPE, NULL);
 	GtkTreeIter iter;
-	PreferencesDCCPage *page = g_new0 (PreferencesDCCPage, 1);
 	PreferencesDialog *p = (PreferencesDialog *) prefs_dialog;
 
 #define GW(name) ((page->name) = glade_xml_get_widget (xml, #name))
@@ -137,10 +140,11 @@ preferences_page_dcc_new (gpointer prefs_dialog, GladeXML *xml)
 #undef GW
 
 	gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (page->download_dir_button), prefs.dccdir);
-	if (strlen (prefs.dcc_completed_dir) == 0)
+	if (strlen (prefs.dcc_completed_dir) == 0) {
 		gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (page->completed_dir_button), prefs.dccdir);
-	else
+	} else {
 		gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (page->completed_dir_button), prefs.dcc_completed_dir);
+	}
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (page->convert_spaces), prefs.dcc_send_fillspaces);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (page->save_nicknames_dcc), prefs.dccwithnick);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (page->autoaccept_dcc_chat), prefs.autodccchat);
@@ -176,19 +180,32 @@ preferences_page_dcc_new (gpointer prefs_dialog, GladeXML *xml)
 	g_signal_connect (G_OBJECT (page->individual_receive_throttle), "value-changed",     G_CALLBACK (irt_changed),                page);
 	g_signal_connect (G_OBJECT (page->global_receive_throttle),     "value-changed",     G_CALLBACK (grt_changed),                page);
 
-	if (g_file_test ("../../data/dcc.png", G_FILE_TEST_EXISTS))
-		page->icon = gdk_pixbuf_new_from_file ("../../data/dcc.png", NULL);
-	else
-		page->icon = gdk_pixbuf_new_from_file (XCHATSHAREDIR "/dcc.png", NULL);
+	GtkIconTheme *theme = gtk_icon_theme_get_default ();
+	PREFERENCES_PAGE (page)->icon = gtk_icon_theme_load_icon (theme, "xchat-gnome-dcc", 16, 0, NULL);
+
 	gtk_list_store_append (p->page_store, &iter);
-	gtk_list_store_set (p->page_store, &iter, 0, page->icon, 1, _("File Transfers & DCC"), 2, 3, -1);
+	gtk_list_store_set (p->page_store, &iter, 0, PREFERENCES_PAGE (page)->icon, 1, _("File Transfers & DCC"), 2, 3, -1);
 
 	return page;
 }
 
-void
-preferences_page_dcc_free (PreferencesDCCPage *page)
+static void
+preferences_page_dcc_init (PreferencesPageDCC *page)
 {
-	g_object_unref (page->icon);
-	g_free (page);
+}
+
+static void
+preferences_page_dcc_dispose (GObject *object)
+{
+	//PreferencesPageDCC *page = (PreferencesPageDCC *) object;
+
+	G_OBJECT_CLASS (preferences_page_dcc_parent_class)->dispose (object);
+}	
+
+static void
+preferences_page_dcc_class_init (PreferencesPageDCCClass *klass)
+{
+	GObjectClass *object_class = (GObjectClass *) klass;
+
+	object_class->dispose = preferences_page_dcc_dispose;
 }
