@@ -1,7 +1,7 @@
 /*
  * preferences-page-spellcheck.c - helpers for the spellcheck preferences page
  *
- * Copyright (C) 2004-2006 xchat-gnome team
+ * Copyright (C) 2004-2007 xchat-gnome team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,8 +27,11 @@
 #include <libsexy/sexy-spell-entry.h>
 #include "preferences-page-spellcheck.h"
 #include "preferences-dialog.h"
+#include "util.h"
 #include "gui.h"
 
+G_DEFINE_TYPE(PreferencesPageSpellcheck, preferences_page_spellcheck, PREFERENCES_PAGE_TYPE)
+
 enum
 {
 	COL_LANG_ACTIVATED = 0,
@@ -37,7 +40,7 @@ enum
 };
 
 static void
-enabled_changed (GtkToggleButton *button, PreferencesSpellcheckPage *page)
+enabled_changed (GtkToggleButton *button, PreferencesPageSpellcheck *page)
 {
 	GConfClient *client;
 	gboolean enabled;
@@ -52,7 +55,7 @@ enabled_changed (GtkToggleButton *button, PreferencesSpellcheckPage *page)
 }
 
 static void
-language_changed (GtkCellRendererToggle *toggle, gchar *pathstr, PreferencesSpellcheckPage *page)
+language_changed (GtkCellRendererToggle *toggle, gchar *pathstr, PreferencesPageSpellcheck *page)
 {
 	GtkTreePath *path;
 	GtkTreeIter iter;
@@ -72,32 +75,27 @@ language_changed (GtkCellRendererToggle *toggle, gchar *pathstr, PreferencesSpel
 		if (activated) {
 			/* we want to deactive it */
 			sexy_spell_entry_deactivate_language (SEXY_SPELL_ENTRY (gui.text_entry), lang);
-		}
-		else {
+		} else {
 			/* we want to activate it */
 			result = sexy_spell_entry_activate_language (SEXY_SPELL_ENTRY (gui.text_entry), lang, &err);
 		}
 
 		if (result) {
 			/* ok we can update the configuration */
-			/*gtk_list_store_set (page->spellcheck_store, &iter,
-			                    COL_LANG_ACTIVATED, !activated,
-			                    -1);
-			*/
 			client = gconf_client_get_default ();
 			languages = sexy_spell_entry_get_active_languages (SEXY_SPELL_ENTRY (gui.text_entry));
 			gconf_client_set_list (client, "/apps/xchat/spellcheck/languages",
 			                       GCONF_VALUE_STRING, languages, NULL);
 
-			if (languages == NULL)
+			if (languages == NULL) {
 				/* No more languages selected: desactivate spellchecking */
 				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (page->enable_spellcheck), FALSE);
+			}
 
 			g_slist_foreach (languages, (GFunc) g_free, NULL);
 			g_slist_free (languages);
 			g_object_unref (client);
-		}
-		else {
+		} else {
 			g_printerr (_("Error in language %s activation: %s\n"), lang, err->message);
 			g_error_free (err);
 		}
@@ -133,8 +131,9 @@ gconf_languages_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry,
 	 * */
 	new_languages = gconf_client_get_list (client, "/apps/xchat/spellcheck/languages", GCONF_VALUE_STRING, NULL);
 
-	if (new_languages != NULL)
+	if (new_languages != NULL) {
 		sexy_spell_entry_set_active_languages (SEXY_SPELL_ENTRY (gui.text_entry), new_languages, &err);
+	}
 
 	if (err) {
 		g_printerr (_("Error in spell checking configuration: %s\n"), err->message);
@@ -147,8 +146,7 @@ gconf_languages_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry,
 			g_slist_foreach (old_languages, (GFunc) g_free, NULL);
 			g_slist_free (old_languages);
 		}
-	}
-	else {
+	} else {
 		/* Then we update the store */
 		if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (store), &iter)) {
 			do {
@@ -166,24 +164,21 @@ gconf_languages_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry,
 	g_slist_free (new_languages);
 }
 
-PreferencesSpellcheckPage *
+PreferencesPageSpellcheck *
 preferences_page_spellcheck_new (gpointer prefs_dialog, GladeXML *xml)
 {
-	PreferencesSpellcheckPage *page = g_new0 (PreferencesSpellcheckPage, 1);
+	PreferencesPageSpellcheck *page = g_object_new (PREFERENCES_PAGE_SPELLCHECK_TYPE, NULL);
 	PreferencesDialog *p = (PreferencesDialog *) prefs_dialog;
-	GtkTreeIter iter;
 	gboolean enabled;
-	GtkTreeViewColumn *column;
 	GSList *languages, *l;
 	GtkWidget *contents_vbox, *page_vbox, *label, *swin;
 	page_vbox = glade_xml_get_widget (xml, "spell check");
 
-	if (g_file_test ("../../data/spellcheck.png", G_FILE_TEST_EXISTS))
-		page->icon = gdk_pixbuf_new_from_file ("../../data/spellcheck.png", NULL);
-	else
-		page->icon = gdk_pixbuf_new_from_file (XCHATSHAREDIR "/spellcheck.png", NULL);
+	PREFERENCES_PAGE (page)->icon = gtk_widget_render_icon (page_vbox, GTK_STOCK_SPELL_CHECK, GTK_ICON_SIZE_MENU, NULL);
+
+	GtkTreeIter iter;
 	gtk_list_store_append (p->page_store, &iter);
-	gtk_list_store_set (p->page_store, &iter, 0, page->icon, 1, _("Spell checking"), 2, 7, -1);
+	gtk_list_store_set (p->page_store, &iter, 0, PREFERENCES_PAGE (page)->icon, 1, _("Spell checking"), 2, 7, -1);
 
 	languages = sexy_spell_entry_get_languages (SEXY_SPELL_ENTRY (gui.text_entry));
 	if (languages == NULL) {
@@ -218,6 +213,7 @@ preferences_page_spellcheck_new (gpointer prefs_dialog, GladeXML *xml)
 	gtk_tree_view_set_model (GTK_TREE_VIEW (page->spellcheck_list),
 	                         GTK_TREE_MODEL (page->spellcheck_store));
 
+	GtkTreeViewColumn *column;
 	page->activated_renderer = gtk_cell_renderer_toggle_new ();
 	g_object_set (G_OBJECT (page->activated_renderer), "activatable", TRUE, NULL);
 	column = gtk_tree_view_column_new_with_attributes (_("Enable"), page->activated_renderer,
@@ -250,13 +246,13 @@ preferences_page_spellcheck_new (gpointer prefs_dialog, GladeXML *xml)
 			                    COL_LANG_CODE, l->data,
 			                    -1);
 			g_free (name);
-		}
-		else
+		} else {
 			gtk_list_store_set (page->spellcheck_store, &iter,
 			                    COL_LANG_NAME, l->data,
 			                    COL_LANG_ACTIVATED, active,
 			                    COL_LANG_CODE, l->data,
 			                    -1);
+		}
 	}
 	g_slist_free (languages);
 
@@ -267,21 +263,47 @@ preferences_page_spellcheck_new (gpointer prefs_dialog, GladeXML *xml)
 	return page;
 }
 
-void
-preferences_page_spellcheck_free (PreferencesSpellcheckPage *page)
+static void
+preferences_page_spellcheck_init (PreferencesPageSpellcheck *page)
 {
+}
+
+static void
+preferences_page_spellcheck_dispose (GObject *object)
+{
+	PreferencesPageSpellcheck *page = (PreferencesPageSpellcheck *) object;
+
+	if (page->spellcheck_store) {
+		g_object_unref (page->spellcheck_store);
+		page->spellcheck_store = NULL;
+	}
+ 
+	G_OBJECT_CLASS (preferences_page_spellcheck_parent_class)->dispose (object);
+}	
+
+static void
+preferences_page_spellcheck_finalize (GObject *object)
+{
+	PreferencesPageSpellcheck *page = (PreferencesPageSpellcheck *) object;
 	gint i;
-	 GConfClient *client;
+	GConfClient *client;
 
 	client = gconf_client_get_default ();
-	for (i = 0; i < 2; i++)
+	for (i = 0; i < 2; i++) {
 		gconf_client_notify_remove (client, page->nh[i]);
+	}
 	g_object_unref (client);
 
-	g_object_unref (page->icon);
-	if (page->spellcheck_store)
-		g_object_unref (page->spellcheck_store);
-	g_free (page);
+	G_OBJECT_CLASS (preferences_page_spellcheck_parent_class)->finalize (object);
+}
+
+static void
+preferences_page_spellcheck_class_init (PreferencesPageSpellcheckClass *klass)
+{
+	GObjectClass *object_class = (GObjectClass *) klass;
+
+	object_class->dispose = preferences_page_spellcheck_dispose;
+	object_class->finalize = preferences_page_spellcheck_finalize;
 }
 
 #endif
