@@ -1,7 +1,7 @@
 /*
  * util.c: Helper functions for miscellaneous tasks
  *
- * Copyright (C) 2004-2006 xchat-gnome team
+ * Copyright (C) 2004-2007 xchat-gnome team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,6 +25,7 @@
 #include <string.h>
 #include "util.h"
 #include "gui.h"
+#include "../common/xchatc.h"
 
 void
 error_dialog (const gchar *header, const gchar *message)
@@ -37,45 +38,12 @@ error_dialog (const gchar *header, const gchar *message)
 					 GTK_BUTTONS_OK,
 					 "%s", header);
 
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", message);
+	gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (dialog), "%s", message);
 
 	gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 }
 
-gint gtk_tree_iter_sort_func_nocase (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer data)
-{
-	gchar *as, *bs;
-	gint result;
-
-	gtk_tree_model_get (model, a, 1, &as, -1);
-	gtk_tree_model_get (model, b, 1, &bs, -1);
-
-	if (as == NULL) return 1;
-	if (bs == NULL) {
-		g_free (as);
-		return -1;
-	}
-
-	result = strcasecmp (as, bs);
-
-	g_free (as);
-	g_free (bs);
-
-	/* GtkTreeSortable has undefined results if this function isn't
-	 * reflexive, antisymmetric and transitive.  If the two strings are
-	 * equal, compare session pointers */
-	if (result == 0) {
-		gpointer ap, bp;
-		gtk_tree_model_get (model, a, 2, &ap, -1);
-		gtk_tree_model_get (model, b, 2, &bp, -1);
-
-		return (GPOINTER_TO_UINT(ap) - GPOINTER_TO_UINT(bp));
-	}
-
-	return result;
-}
-
 gboolean
 dialog_escape_key_handler_destroy (GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
@@ -111,10 +79,11 @@ menu_position_under_widget (GtkMenu *menu, gint *x, gint *y, gboolean *push_in,
 	gdk_window_get_origin (w->window, x, y);
 	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
 
-	if (gtk_widget_get_direction (w) == GTK_TEXT_DIR_RTL)
+	if (gtk_widget_get_direction (w) == GTK_TEXT_DIR_RTL) {
 		*x += w->allocation.x + w->allocation.width - requisition.width;
-	else
+	} else {
 		*x += w->allocation.x;
+	}
 
 	*y += w->allocation.y + w->allocation.height;
 
@@ -160,3 +129,33 @@ menu_position_under_tree_view (GtkMenu *menu, gint *x, gint *y, gboolean *push_i
 		menu_position_under_widget (menu, x, y, push_in, tree);
 	}
 }
+
+gchar *
+locate_data_file (const gchar *file_name)
+{
+	gchar *uninstalled_path, *path;
+
+	uninstalled_path = g_build_filename (TOPSRCDIR, "data", file_name, NULL);
+	if (g_file_test (uninstalled_path, G_FILE_TEST_EXISTS)) {
+		path = uninstalled_path;
+	} else {
+		g_free (uninstalled_path);
+		path = gnome_program_locate_file (gnome_program_get (), GNOME_FILE_DOMAIN_APP_DATADIR, file_name, FALSE, NULL);
+	}
+
+	g_return_val_if_fail (path != NULL, NULL);
+
+	return path;
+}
+
+server *
+find_connected_server (ircnet *network)
+{
+	for (GSList *i = sess_list; i; i = g_slist_next (i)) {
+		session *sess = (session *)(i->data);
+		if (sess->server->network == network) {
+			return sess->server;
+		}
+	}
+	return NULL;
+}
